@@ -185,7 +185,7 @@ pivot <- function(data,
             # value_col <- .Call(C_pivot_long, vd, cc, FALSE) # Csv(c_to_vec(data[values]), cc_vec)
             # variable_col <- rep(if(factor[1L]) seq_along(values) else nam[values], vlengths(cc))
           } else {
-            id_cols <- .Call(C_rbindlist, alloc(data[ids], length(values)), FALSE, FALSE, NULL) # .Call(C_subsetDT, data, rep.int(seq_len(n), length(values)), ids, FALSE)
+            id_cols <- .Call(C_rbindlist, alloc(data[ids], length(values), FALSE), FALSE, FALSE, NULL) # .Call(C_subsetDT, data, rep.int(seq_len(n), length(values)), ids, FALSE)
             # This is faster than .Call(C_pivot_long, vd, NULL) because rep() is slow...
             value_cols <- .Call(C_pivot_long, vd, NULL, TRUE) # .Call(C_rbindlist, lapply(vd, list), FALSE, FALSE, "id")
             # value_col <- .Call(C_pivot_long, vd, NULL)   # c_to_vec(data[values])
@@ -272,7 +272,7 @@ pivot <- function(data,
           g <- g[[2L]]
           attr(g, "N.groups") <- ng
         } else { # Could also use GRP(), but this avoids computing a potentially large and redundant group sizes vector
-          g <- group(data[ids], starts = TRUE)
+          g <- groupv(data[ids], starts = TRUE)
           id_cols <- .Call(C_subsetDT, data, attr(g, "starts"), ids, FALSE)
         }
         # (4) Compute Names and Labels Columns
@@ -308,7 +308,7 @@ pivot <- function(data,
           namv <- names(data)[values]
           attributes(data) <- NULL
           if(!is.character(FUN)) {
-            data[values] <- apply_external_FUN(data[values], group(list(g, g_v)), FUN, FUN.args, l1orlst(as.character(substitute(FUN))))
+            data[values] <- apply_external_FUN(data[values], group(g, g_v), FUN, FUN.args, l1orlst(as.character(substitute(FUN))))
             FUN <- "last"
           }
           value_cols <- lapply(data[values], function(x) .Call(C_pivot_wide, g, g_v, x, fill, nthreads, FUN, na.rm))
@@ -318,7 +318,7 @@ pivot <- function(data,
           names(value_cols) <- if(transpose[1L]) namv_res else t(namv_res)
         } else {
           if(!is.character(FUN)) {
-            data[[values]] <- apply_external_FUN(data[[values]], group(list(g, g_v)), FUN, FUN.args, l1orlst(as.character(substitute(FUN))))
+            data[[values]] <- apply_external_FUN(data[[values]], group(g, g_v), FUN, FUN.args, l1orlst(as.character(substitute(FUN))))
             FUN <- "last"
           }
           value_cols <- .Call(C_pivot_wide, g, g_v, data[[values]], fill, nthreads, FUN, na.rm)
@@ -358,7 +358,7 @@ pivot <- function(data,
             g <- g[[2L]]
             attr(g, "N.groups") <- ng
           } else { # Could also use GRP(), but this avoids computing a potentially large and redundant group sizes vector
-            g <- group(data[ids], starts = TRUE)
+            g <- groupv(data[ids], starts = TRUE)
             id_cols <- .Call(C_subsetDT, data, attr(g, "starts"), ids, FALSE)
           }
         } else {
@@ -398,11 +398,11 @@ pivot <- function(data,
           attributes(vd) <- NULL
         }
         if(!is.character(FUN)) {
-          vd <- apply_external_FUN(vd, group(list(g, g_v)), FUN, FUN.args, l1orlst(as.character(substitute(FUN))))
+          vd <- apply_external_FUN(vd, group(g, g_v), FUN, FUN.args, l1orlst(as.character(substitute(FUN))))
           FUN <- "last"
         }
         value_cols <- lapply(vd, function(x) .Call(C_pivot_wide, g, g_v, x, fill, nthreads, FUN, na.rm))
-        if(length(id_cols)) id_cols <- .Call(C_rbindlist, alloc(id_cols, length(value_cols)), FALSE, FALSE, NULL)
+        if(length(id_cols)) id_cols <- .Call(C_rbindlist, alloc(id_cols, length(value_cols), FALSE), FALSE, FALSE, NULL)
         value_cols <- .Call(C_rbindlist, value_cols, FALSE, FALSE, names[[2L]]) # Final column is "variable" name
 
         names(value_cols) <- c(names[[2L]], names1)
